package com.iflytek.cloud;

import android.os.Parcel;
import android.os.Parcelable;

/**
 * <h3>语义结果</h3>
 * <p>
 * 		语义结果为文本内容，格式包括xml结构化文本(xml)、json结构化文本(json)2种。
 * </p>
 * <p>
 * 		若{@link SpeechConstant#NLP_VERSION}值为1.0，则只支持xml格式。
 * </p>
 * <p>
 * 		关于json和xml格式的各关键字说明，请参考<script>gShowUserGuideFile();</script>。关于语义理解结果的
 * 详细说明，请参考<script>gShowUnderstanderFile();</script>。
 * </p>
 * 
 * @since version:1073
 * @author <script>gShowAuthor();</script>
 * @see TextUnderstander
 * @see SpeechUnderstander
 * @see SpeechConstant#RESULT_TYPE
 * @see SpeechConstant#NLP_VERSION
 * @see SpeechUnderstanderListener
 * @see TextUnderstanderListener
 */
public class UnderstanderResult implements Parcelable {
	/**
	 * 语义理解结果，请参考语义说明文档。
	 */
	private String mXml = "";

	/**
	 * <h4>构造函数</h4>
	 * <p>
	 * 		应用层只通过{@link SpeechUnderstanderListener#onResult(UnderstanderResult)}或
	 * {@link TextUnderstanderListener#onResult(UnderstanderResult)}得到结果，可忽略此
	 * 的构造函数。
	 * </p>
	 * @param in 文本结果
	 * 
	 * @hide
	 */
	public UnderstanderResult(Parcel in) {
	    mXml = in.readString();
    }

	/**
	 * <h4>构造函数</h4>
	 * <p>
	 * 		应用层只通过{@link SpeechUnderstanderListener#onResult(UnderstanderResult)}或
	 * {@link TextUnderstanderListener#onResult(UnderstanderResult)}得到结果，可忽略此
	 * 的构造函数。
	 * </p>
	 * @param xml 文本结果
	 */
	public UnderstanderResult(String xml) {
		if (null != xml) {
			this.mXml = xml;
		}
	}
	
	/**
	 * <h4>获取识别结果</h4>
	 * @return String类型识别结果文本。返回的结果文本有可能为空字符串。关于语义理解结果的
	 * 详细说明，请参考<script>gShowUnderstanderFile();</script>。
	 */
	public String getResultString() {
		return mXml;
	}

	/**
	 * @hide
	 */
	public int describeContents() {
		return 0;
	}

    /**
     * @hide
     */
    public void writeToParcel(Parcel dest, int flags) { 
        dest.writeString(mXml);
    } 
    
    /**
     * @hide
     */
	public static final Parcelable.Creator<UnderstanderResult> CREATOR = new Parcelable.Creator<UnderstanderResult>() {

		@Override
		public UnderstanderResult createFromParcel(Parcel in) {
			return new UnderstanderResult(in);
		}

		@Override
		public UnderstanderResult[] newArray(int size) {
			return new UnderstanderResult[size];
		}
	};
}
