package com.iflytek.cloud.msc.module;

import android.text.TextUtils;

import java.util.HashSet;
import java.util.Iterator;

import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechUtility;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.param.HashParam;
import com.iflytek.msc.MSC;
import com.iflytek.speech.ISpeechModule;

public abstract class SpeechInterface {
	protected static final Object sSync = new Object();
	
	// 会话参数
	protected HashParam mSessionParams = new HashParam();
	
	protected SpeechInterface(){
	}
	
	public enum ENGINE_MODE
	{
		MSC	// 使用MSC
		//PLUS,	// 使用语记
		//AUTO	// 自动选择
	}
	/**
	 * 设置识别参数。
	 * <p>设置的参数值在下次识别仍然有效。<br>
	 * 包括引擎类型、语言、语言区域、领域、前后端点超时等单独的参数。
	 * 包括Params的组合参数。
	 * @param key
	 *            参数名称
	 * @param value
	 *            参数值
	 * @return 错误码，0表示成功
	 */
	public boolean setParameter(String key, String value) {
		if (TextUtils.isEmpty(key)) {
			return false;
		}
		
		if(key.equals(SpeechConstant.PARAMS)){
			if(TextUtils.isEmpty(value))
				mSessionParams.clear();
			else
				mSessionParams.putMultiParam(value);
			return true;
		}
		
		if (TextUtils.isEmpty(value)) {
			return mSessionParams.removeParam(key);
		}
			
		mSessionParams.putParam(key, value);
		return true;
	}
	
	/**
	 * 设置参数列表，复数形式，将清除已设置参数
	 * @return true->成功，false->失败
	 */
	public boolean setParameter(HashParam params)
	{
		mSessionParams = params.clone();
		return true;
	}
	
	/**
	 * 获取识别参数。
	 * <p>包括引擎类型、语言、语言区域、领域、前后端点超时等，另外包括支持的语言列表等。
	 * @param key
	 *            参数名称
	 * @return 参数值
	 */
	public String getParameter(String key) {
		if(SpeechConstant.PARAMS.equals(key))
			return mSessionParams.toString();
		else
			return mSessionParams.getString(key);
	}
	
	/**
	 * 返回引擎的选择模式，是使用MSC还是语记
	 */
	protected ENGINE_MODE getStartMode(String eng, ISpeechModule binder)
	{
		// @ { start: 去除语记支持
		/*// 支持每个业务去设置engineMode
		String mode = getParameter(SpeechConstant.ENGINE_MODE);
		if(SpeechConstant.MODE_MSC.equals(mode))
			return ENGINE_MODE.MSC;
		else if(SpeechConstant.MODE_PLUS.equals(mode))
			return ENGINE_MODE.PLUS;
		
		SpeechUtility utility = SpeechUtility.getUtility();
		// 未初始化，返回错误
		if(utility == null)
			return ENGINE_MODE.MSC;
		// 如果用户指定加载模式，直接返回
		if(utility.getEngineMode() != ENGINE_MODE.AUTO)
			return utility.getEngineMode();
		
		*//**** 以下是AUTO模式  ****//*
		// 如果MSC库未加载，返回语记模式
		if(!MSC.isLoaded())
			return ENGINE_MODE.PLUS;
		
		String engineType = getParameter(SpeechConstant.ENGINE_TYPE);
		
		if(SpeechConstant.TYPE_LOCAL.equals(engineType))
		{
			//离线：使用语记
			return ENGINE_MODE.PLUS;
		}else if(SpeechConstant.TYPE_MIX.equals(engineType))
		{
			//混合：服务可用使用语记、不可用使用MSC
			if(binder == null || !binder.isAvailable())
				return ENGINE_MODE.MSC;
			else
				return ENGINE_MODE.PLUS;
		}else {
			//在线：使用MSC
			return ENGINE_MODE.MSC;
		}*/

		return ENGINE_MODE.MSC;
		// end: @}
	}

	public boolean destroy(){
		return true;
	}
}
