package com.iflytek.cloud.msc.tts;

import java.io.IOException;
import java.util.ArrayList;

import android.content.Context;
import android.media.AudioManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.text.TextUtils;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.SpeechEvent;
import com.iflytek.cloud.SynthesizerListener;
import com.iflytek.cloud.msc.module.SpeechModule;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.param.HashParam;
import com.iflytek.cloud.param.MscKeys;
import com.iflytek.cloud.record.PcmBuffer;
import com.iflytek.cloud.record.PcmPlayer;
import com.iflytek.cloud.record.PcmPlayer.PcmPlayerListener;


/**
 * 合成播放对象,用来调度MSC及播放器
 * @author yjzhao
 *
 */
public class SpeakSession extends SpeechModule{
	
	public abstract interface BufferReadyListener
	{
		void onBufferReady();
	}
	
	// 播放器
	private PcmPlayer mPlayer = null;
	// 音频缓存
	private PcmBuffer mBuffer = null;
	// 播放回调接口
	private SynthesizerListener mListener = null;
	
	// 合成到文件的回调接口
	private SynthesizerListener mUriListener = null;
	
	// 合成完成的通知回调接口
	private BufferReadyListener mReadyListener = null;
	
	// 缓冲播放时间，单位：ms
	private int minPlaySec = 0;
	// 是否已经开始播放
	private boolean playBegin  = false;
	// 是否已经开始合成
	public boolean synthesizeBegin  = false;
	
	public boolean bufferComplete = false;
	
	
	// 播放文本
	public String mText = null;
	// 是否发生错误,主要用来判断是否可以支持预合成的播放请求
	public SpeechError mError = null;
	
	public SpeakSession(Context context) {
		super(context);
	}
	
	/**
	 * 设置预合成的内容
	 */
	public void setBufferText(String text,HashParam params)
	{
		setParameter(params);
		mText = text;
	}
	/**
	 * 开始预合成
	 */
	public void startBufferText()
	{
		if(bufferComplete)
			return;
		
		final String local_save_path = mSessionParams.getString(MscKeys.KEY_TTS_NEXT_AUDIO_PATH);
		startSpeaking(mText, mSessionParams, null,false, local_save_path);
	}
	
	/**
	 * 设置缓冲完成的外部通知接口
	 * @param listener
	 */
	public void setBufferReadyLietener(BufferReadyListener listener)
	{
		mReadyListener = listener;
	}
	
	/**
	 * 开始播放
	 * @param text
	 * @param params
	 * @param listener
	 * @param playSound 是否直接播放
	 */
	public synchronized int startSpeaking(String text,HashParam params,SynthesizerListener listener,boolean playSound, final String savePath)
	{
		int ret = ErrorCode.SUCCESS;
		
		try{
			DebugLog.LogD("tts start:" + System.currentTimeMillis());
			mListener = listener;
			mText = text;
			setParameter(params);
			int streamType = params.getInt(SpeechConstant.STREAM_TYPE
					, AudioManager.STREAM_MUSIC);
			boolean request = params.getBoolean(SpeechConstant.KEY_REQUEST_FOCUS
					, true);
			
			if(playSound){
				final String strBufFading = mSessionParams.remove(MscKeys.KEY_TTS_BUFFERING_FADING );
				final boolean bufferingFadingEnable = HashParam.parseBoolean( strBufFading, false );
				
				final String strFading = mSessionParams.remove( SpeechConstant.TTS_FADING );
				final boolean fadingEnable = HashParam.parseBoolean( strFading, false);
				mPlayer = new PcmPlayer( mContext, streamType, request, fadingEnable, bufferingFadingEnable );
			}
			
			mscer = new MscSynthesizer(mContext,params,getHandlerThread("tts"));

			final int  minAudioSecs = Math.max( 1, params.getInt(MscKeys.KEY_TTS_MIN_AUDIO_LEN, 0)/1000 );
			final int textLen = null!=text ? text.length() : 0;
			
			final int procScale = params.getInt( MscKeys.KEY_TTS_PROC_SCALE, PcmBuffer.DEF_PROC_SCALE );
			mBuffer = new PcmBuffer(mContext, mscer.getSampleRate(), minAudioSecs+textLen ,savePath, procScale);
			
			final String strEndWithNull = mSessionParams.remove( MscKeys.END_WITH_NULL );
			mBuffer.setEndWithNull( HashParam.parseBoolean(strEndWithNull, true) );
			

			minPlaySec = params.getInt(SpeechConstant.TTS_BUFFER_TIME,0);
			DebugLog.LogD("minPlaySec:" + minPlaySec);
			playBegin  = false;
			
			((MscSynthesizer)mscer).startSyn(text,synListener);
			synthesizeBegin = true;
		}catch(SpeechError e){
			ret = e.getErrorCode();
			DebugLog.LogE( e );
		}catch( Throwable e ){
			ret = ErrorCode.ERROR_UNKNOWN;
			DebugLog.LogE( e );
		}//end of try-catch
		
		return ret;
	}
	
	public void setListener(SynthesizerListener listener)
	{
		mListener = listener;
	}
	
	/**
	 * 获取合成进度
	 * @return
	 */
	public int getState() {
		if(mBuffer != null && mPlayer != null)
			return mPlayer.getState();
		return PcmPlayer.STOPED;
	}
	
	/**
	 * 判断是否预合成的对象，是否可以直接播放
	 */
	public boolean isBufferPlayable(String nextText)
	{
		if(mBuffer != null && mBuffer.readyToPlay(0)
			&& mError == null
			&& !TextUtils.isEmpty(nextText) 
			&& nextText.equals(mText) )
			return true;
		return false;
	}
	
	/**
	 * 暂停方法
	 */
	public void pauseSpeaking() {
		if(mBuffer != null && mPlayer != null)
		{
			mPlayer.pause();
		}
	}
	
	public void replaySpeaking() {
		DebugLog.LogD("Session replay");
		if(mBuffer != null && mPlayer != null && (PcmPlayer.STOPED == mPlayer.getState()))
		{
			// 重播事件
			mPlayer.rePlay(mBuffer, playListener);
		}
	}
	
	/**
	 * 根据当前会话是否活动，判断新的会话是否可以开启
	 * @return
	 */
	public boolean isSpeaking()
	{
		if(isRunning())
			return true;
		if(getState() != PcmPlayer.STOPED 
				&& getState() != PcmPlayer.INIT)
			return true;
		return false;
	}
	
	/**
	 * 开始播放
	 */
	private void startPlay()
	{
		if(!playBegin && mPlayer !=null && mBuffer.readyToPlay(minPlaySec))
		{
			playBegin = true;
			mPlayer.play(mBuffer, playListener);
			if (null != mListener)
				Message.obtain(mUiHandler, MSG_BEGIN).sendToTarget();
		}
	}

	/**
	 * 恢复播放
	 */
	public void resumeSpeaking() 
	{
		if(mBuffer != null && mPlayer != null)
		{
			// 恢复播放
			mPlayer.resume();
		}else
		{
			// 创建播放器
			mPlayer = new PcmPlayer(mContext);
			startPlay();
		}
	}
	
	/**
	 * 取消事件，需要保存消息，有可能会话已经结束，仍需要保存
	 */
	public void cancel(boolean notifyError) 
	{
		DebugLog.LogD("SpeakSession cancel notifyError:"+notifyError);				
		//系统合成接口的等待锁mCondition必须通过回调来解锁，所以在这里加一个onEnvent回调
		if(isSpeaking())
		{
			if(mListener != null)
				mListener.onEvent(SpeechEvent.EVENT_TTS_CANCEL, 0, 0, null);
			if(mUriListener != null)
				mUriListener.onEvent(SpeechEvent.EVENT_TTS_CANCEL, 0, 0, null);

			if(notifyError)
			{
				final SpeechError error = new SpeechError(ErrorCode.ERROR_INTERRUPT);
				if (mListener != null) {
					DebugLog.LogD("tts-onCompleted-cancel");
					Message.obtain(mUiHandler, MSG_COMPLETE, error).sendToTarget();
				}

				if (mUriListener != null) {
					if (message_main_thread) {
						Message.obtain(mUriHandler, MSG_COMPLETE, error).sendToTarget();
					}else {
						mUriListener.onCompleted(error);
					}
				}
			}//end of if notify error
		}
		mListener = null;
		mUriListener = null;
		super.cancel(false);
		if (mPlayer != null)
			mPlayer.stop();
//		if(mBuffer != null)
//			mBuffer.deleteFile();
	}
	
	
	@Override
	public boolean destroy() {
		synchronized (mSynObj) {
			cancel(false);
			return true;	
		}
	}




	/**
	 * 合成回调监听器
	 */
	private MscSynthesizerListener synListener = new MscSynthesizerListener() {

		/**
		 * 合成缓冲进度回调
		 * @param  datas 录音数据回调
		 * @param percent 合成缓冲进度回调
		 * @param beginPos 合成起始端位置
		 * @param endPos 合成结束位置
		 * @param spellInfo 合成返回的拼音标注
		 */
		public void onBufferProgress(ArrayList<byte[]> datas, int percent
				, int beginPos, int endPos, String spellInfo) {
			Bundle bundle = new Bundle();
			bundle.putInt(KEY_PERCENT, percent);
			bundle.putInt(KEY_BEGPOS, beginPos);
			bundle.putInt(KEY_ENDPOS, endPos);
			bundle.putString(KEY_SPELL_INFO, spellInfo);
			
			if (null != mListener) {
				Message.obtain(mUiHandler, MSG_BUFFER_PERCENT, bundle).sendToTarget();
			}
			
			// write to buffer
			try {
				mBuffer.writeStream(datas, percent,beginPos,endPos);
				
				// 开始播放
				startPlay();
			} catch (IOException e) {
				DebugLog.LogE(e);
				mError = new SpeechError(ErrorCode.ERROR_FILE_ACCESS);
				Message.obtain(mUiHandler, MSG_COMPLETE, 
						mError).sendToTarget();
				cancel(false);
			}
		}

		/**
		 * 合成结束回调
		 */
		public void onCompleted(SpeechError error) {
			//DebugLog.LogD("-->onCompleted"+mscer+" spearksession:"+SpeakSession.this);
			mError = error;
			if(error == null)
			{
				bufferComplete = true;
				// 保存到本地文件中,只有设置文件路径生效
				String format = null;
				if(mscer != null)
				{
					format =  mscer.getParam().getString(SpeechConstant.AUDIO_FORMAT, null);
				}
				mBuffer.renameToLocal(format);
				
				// 通知外部进行预合成
				if(mReadyListener != null)
				{
					mReadyListener.onBufferReady();
					DebugLog.LogD("onCompleted NextSession pause");
				}
			}
			
			// 通过onEvent抛出服务端会话id
			Bundle bundle = new Bundle();
			bundle.putString(SpeechEvent.KEY_EVENT_SESSION_ID, mscer.getSessionID());
			Message.obtain(mUiHandler, MSG_EVENT, bundle).sendToTarget();

			if (null != mListener && error != null)
			{
				Message.obtain(mUiHandler, MSG_COMPLETE, error).sendToTarget();
				if(mPlayer != null)
					mPlayer.stop();
			}
		}
	};

	/**
	 * 播放监听器
	 */
	private PcmPlayerListener playListener = new PcmPlayerListener() {

		/**
		 * 暂停回调
		 */
		public void onPaused() {
			
			if (null != mListener)
			{
				Message.obtain(mUiHandler, MSG_PAUSE).sendToTarget();
			}
			
		}

		/**
		 * 恢复播放
		 */
		public void onResume() {
			// 强制恢复事件
			if (null != mListener)
			{
				Message.obtain(mUiHandler, MSG_RESUME).sendToTarget();
			}
		}

		/**
		 * 返回播放进度
		 */
		public void onPercent(int percent,int beginPos,int endPos) {
			Message.obtain(mUiHandler, MSG_PLAY_PERCENT, percent
					, beginPos, endPos).sendToTarget();
		}

		/**
		 * 合成结束回调
		 */
		public void onError(SpeechError error) {
			// 错误
			Message.obtain(mUiHandler, MSG_COMPLETE, error).sendToTarget();
			if(null != mPlayer)
				mPlayer.stop();
			cancel(false);
		}

		@Override
		public void onStoped() {
			// 播放完成事件
			Message.obtain(mUiHandler, MSG_COMPLETE, null).sendToTarget();
		}

	};
	
	private static final int MSG_BEGIN = 1;
	private static final int MSG_BUFFER_PERCENT = 2;
	private static final int MSG_PAUSE = 3;
	private static final int MSG_RESUME = 4;
	private static final int MSG_PLAY_PERCENT = 5;
	private static final int MSG_COMPLETE = 6;
	private static final int MSG_EVENT = 7;
	
	private static final String KEY_PERCENT = "percent";
	private static final String KEY_BEGPOS = "begpos";
	private static final String KEY_ENDPOS = "endpos";
	private static final String KEY_SPELL_INFO = "spellinfo";
	
	private Handler mUiHandler = new Handler(Looper.getMainLooper()) {
		private int mLastPlayEndPos = 0;
		
		@Override
		public void handleMessage(Message msg) {
			try {
				if(mListener == null)
					return;
				
				switch (msg.what) {
				case MSG_BEGIN:
					DebugLog.LogD("tts-onSpeakBegin");
					mListener.onSpeakBegin();
					break;
				case MSG_BUFFER_PERCENT:
					{
						Bundle bundle = (Bundle)msg.obj;
						int percent = bundle.getInt(KEY_PERCENT);
						int begpos = bundle.getInt(KEY_BEGPOS);
						int endpos = bundle.getInt(KEY_ENDPOS);
						String spell = bundle.getString(KEY_SPELL_INFO);						
						if (null != mListener){
							DebugLog.LogI("tts-onBufferProgress percent: "+percent+", beg: "+begpos+", endpos: "+endpos+", spell: "+spell );
							mListener.onBufferProgress(percent
									, begpos, endpos, spell);
						}
							
					}
					break;
				case MSG_PAUSE:
					DebugLog.LogD("tts-onSpeakPaused");
					mListener.onSpeakPaused();
					break;
				case MSG_RESUME:
					DebugLog.LogD("tts-onSpeakResumed");
					mListener.onSpeakResumed();
					break;
				case MSG_PLAY_PERCENT:
					{	
						int endPos = (Integer)msg.obj;						
						if (null != mListener){
							if( mLastPlayEndPos != endPos ){
								DebugLog.LogD("tts-onSpeakProgress percent: "+msg.arg1+", begpos: "+msg.arg2+", endpos: "+endPos );
								mLastPlayEndPos = endPos;
							}
							
							mListener.onSpeakProgress(msg.arg1, msg.arg2, endPos);
						}
							
					}
					break;
				case MSG_COMPLETE:
					DebugLog.LogD("tts-onCompleted");
					mListener.onCompleted((SpeechError)msg.obj);
					break;
				case MSG_EVENT:
					mListener.onEvent(SpeechEvent.EVENT_SESSION_ID, 0, 0, (Bundle) msg.obj);
				default:
					break;
				}
			} catch (Exception e) {
				DebugLog.LogE("SpeakSession mUiHandler error:"+e);
			}
		}
	};
	
	
	// 是否在主线程回调（语记反馈在主线程回调音频过于频繁会阻塞且导致崩溃）
	private boolean message_main_thread = true;
	/**
	 * 合成到文件的功能
	 * @param text
	 * @param params
	 * @param listener
	 * @return
	 */
	public int synthesizeToUri(String text,final String uri,HashParam params,SynthesizerListener listener) 
	{
		int ret = ErrorCode.SUCCESS;
		
		try{
			message_main_thread =  params.getBoolean(MscKeys.MESSAGE_MAIN_THREAD, true);
			mUriListener = listener;
			mscer = new MscSynthesizer(mContext,params,getHandlerThread( "tts" ));
			final int  minAudioSecs = Math.max( 1, params.getInt(MscKeys.KEY_TTS_MIN_AUDIO_LEN, 0)/1000 );
			final int textLen = null!=text ? text.length() : 0;
			final int procScale = params.getInt( MscKeys.KEY_TTS_PROC_SCALE, PcmBuffer.DEF_PROC_SCALE );
			mBuffer = new PcmBuffer(mContext, mscer.getSampleRate(), minAudioSecs+textLen, uri, procScale);
			
			MscSynthesizerListener synthesizerToUriListener = new MscSynthesizerListener() {
				
				@Override
				public void onCompleted(SpeechError error) {
					//不需要回调完成，onBufferProgress已经回调，不然会重复
					if (null != mUriListener && error != null)
					{
						if (message_main_thread) {
							Message.obtain(mUriHandler, MSG_COMPLETE, error).sendToTarget();
						}else {
							mUriListener.onCompleted(error);
						}
						
					}
				}
				
				@Override
				public void onBufferProgress(ArrayList<byte[]> datas, int progress,
						int beginPos, int endPos, String spellInfo) throws SpeechError {
					
					//对外抛出数据信息，只有tts_data_notify设置了的情况才进行
					if (mscer!= null && mscer.getParam().getBoolean(SpeechConstant.TTS_DATA_NOTIFY, false) 
							&& null != mUriListener && null != datas)
					{
						for(int i = 0; i < datas.size(); i++)
						{
							
							byte[] data = datas.get(i);
							Bundle bundle = new Bundle();
							bundle.putByteArray(SpeechEvent.KEY_EVENT_TTS_BUFFER, data);
							if (message_main_thread) {	
								//构造一个Event Message作为发送Message的obj
								Message msg = Message.obtain();
								msg.what = SpeechEvent.EVENT_TTS_BUFFER;
								msg.arg1 = 0;
								msg.arg2 = 0;
								msg.obj = bundle;
								Message.obtain(mUriHandler, MSG_EVENT, 0, 0,  msg).sendToTarget();
							}
							else {
								mUriListener.onEvent( SpeechEvent.EVENT_TTS_BUFFER, 0, 0, bundle);
							}
						}
					}
					
					try {
						mBuffer.writeStream(datas,progress,beginPos,endPos);
						
						if (null != mUriListener)
						{
							Bundle bundle = new Bundle();
							bundle.putInt(KEY_PERCENT, progress);
							bundle.putInt(KEY_BEGPOS, beginPos);
							bundle.putInt(KEY_ENDPOS, endPos);
							bundle.putString(KEY_SPELL_INFO, spellInfo);
							if (message_main_thread) {	
								Message.obtain(mUriHandler, MSG_BUFFER_PERCENT, bundle).sendToTarget();
							}
							else {
								mUriListener.onBufferProgress(progress, beginPos, endPos, spellInfo);
							}
						}					
						
						if(progress >= 100)
						{
							if(mscer!= null && mscer.getParam().getBoolean(SpeechConstant.TTS_DATA_NOTIFY, false))
							{
								//如果对外抛出音频，uri设置不为null则保存音频，否则不保存音频
								String format = mscer.getParam().getString(SpeechConstant.AUDIO_FORMAT, null);
								
								if(!TextUtils.isEmpty(uri) && mBuffer.renameToLocal(format) != true)
								{
									throw new IOException();
								}
							}
							else
							{
								String format = null;
								if(mscer != null)
								{
									format =  mscer.getParam().getString(SpeechConstant.AUDIO_FORMAT, null);
								}
								if(mBuffer.getMemFileLenth() == 0)
								{
									throw new SpeechError(ErrorCode.MSP_ERROR_NO_DATA);
								}
								
								// 保存到本地文件中,只有设置文件路径生效
								if( mBuffer.renameToLocal(format) != true)
								{
									throw new IOException();
								}
							}
							if( null != mUriListener)
							{
								if (message_main_thread) {
									//如果没有错误，直接回调完成消息
									Message.obtain(mUriHandler, MSG_COMPLETE, null).sendToTarget();
								}else {
									mUriListener.onCompleted(null);
								}
							}
						}
					} catch (IOException e) {
						DebugLog.LogE(e);

						if (null != mUriListener)
						{
							if (message_main_thread) {
								Message.obtain(mUriHandler,MSG_COMPLETE, new SpeechError(ErrorCode.ERROR_FILE_ACCESS)).sendToTarget();
							}else {
								try {
									mUriListener.onCompleted( new SpeechError(ErrorCode.ERROR_FILE_ACCESS));
								} catch (Exception e2) {
									//防止多线程mUriListener为空
								}
								
							}
						}
						// 写入失败，取消
//						cancel(false);
						//TODO 
						if(mscer != null)
						{
							mscer.cancel(false);
						}
						return;
					} 
				}
			};//end of synthesizerToUriListener
			
			((MscSynthesizer)mscer).startSyn(text,synthesizerToUriListener);
		}catch(SpeechError e){
			ret = e.getErrorCode();
			DebugLog.LogE( e );
		}catch( Throwable e ){
			ret = ErrorCode.ERROR_UNKNOWN;
			DebugLog.LogE( e );
		}//end of try-catch
		
		return ret;
	}
	
	final Handler mUriHandler = new Handler(Looper.getMainLooper()) {
		@Override
		public void handleMessage(Message msg) {
			try {
				if(mUriListener == null)
					return;
				switch (msg.what) {
				case MSG_BUFFER_PERCENT:
					Bundle bundle = (Bundle)msg.obj;
					int percent = bundle.getInt(KEY_PERCENT);
					int begpos = bundle.getInt(KEY_BEGPOS);
					int endpos = bundle.getInt(KEY_ENDPOS);
					String spell = bundle.getString(KEY_SPELL_INFO);						
					mUriListener.onBufferProgress(percent, begpos, endpos, spell);
					break;
				case MSG_COMPLETE:
					mUriListener.onCompleted((SpeechError)msg.obj);
					break;
				case MSG_EVENT:
					Message event = (Message) msg.obj;
					if(event != null)
					{
						mUriListener.onEvent(event.what, event.arg1, event.arg2, (Bundle)event.obj);
					}
					break;
				default:
					break;
				}
			} catch (Exception e) {
				DebugLog.LogE("SpeakSession mUiHandler error:"+e);
			}
		}
	};	//end of mUriHandler
	
	/**
	 * 是否正在合成
	 */
	public boolean isRunning(){
		return super.isRunning();
	}
}
