package com.iflytek.cloud.msc.util;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

import android.app.Activity;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationManager;
import android.util.Base64;

import com.iflytek.cloud.Setting;
import com.iflytek.cloud.msc.util.log.DebugLog;

/**
 * 用户信息保存类，保存经纬度、用户更新、使用时间等
 * @author yjzhao
 *
 */
public class Config {
	
	// 纬度
	public static final String  KEY_LATITUDE  = "msc.lat";
	// 经度
	public static final String  KEY_LONGITUDE = "msc.lng";
	
	// 广告上次更新时间，格林尼治时间
	// 检查更新周期为一天，24小时检查一次
	// 上次检查更新时间
	public static final String  KEY_AD_LAST_UPDATE = "ad_last_update";
	
	// GPS上次更新时间
	public static final String  KEY_LOCTION_LAST_UPDATE = "loction_last_update";
	
	// 是否第一次启动
	public static final String KEY_FIRST_START = "first_start";
	
	//用户配置文件名称
	public static final String PREFERENCES_NAME = "com.iflytek.msc";
	
	private SharedPreferences mLocalPreferences = null;
	
	private Context mContext =  null;
	// 单例对象
	public static Config mUserConfig = null;
	

	// 是否具有GPS获取权限
	private boolean hasGpsPermission = true;
	
	// GPS更新时间
	private long loction_Update_Time = 0;
	
	/**
	 * 辅助类初始化，包括广告、配置、数据等
	 * @param context
	 * @return
	 */
	private synchronized static Config initInstance(Context context)
	{
		if(mUserConfig == null)
		{
			mUserConfig = new Config(context);
		}
		// 初始化文件保存路径
		FileUtil.getUserPath(context);
		// 初始化应用信息
		AppInfoUtil.getAppInfo(context);
		return mUserConfig;
	}
	
	/**
	 * 调用此接口前，必须先保证initInstance方法被调用
	 * @param context
	 * @return
	 */
	public static Config getConfig(Context context)
	{
		if(mUserConfig == null && context != null)
		{
			initInstance(context);
		}
		return mUserConfig; 
	}
	
	/**
	 * 初始化用户配置信息
	 * @param context
	 */
	private Config(Context context)
	{
		mContext = context;
		mLocalPreferences = context.getSharedPreferences(PREFERENCES_NAME, Activity.MODE_PRIVATE);
		
		hasGpsPermission = hasGPSPermission(context);
	}
	
	/**
	 * 保存boolean类型值
	 * @param key
	 * @param value
	 */
	public void putBoolean(String key,boolean value)
	{
		Editor localEditor = mLocalPreferences.edit();
		localEditor.putBoolean(key, value);
		localEditor.commit();
	}
	
	/**
	 * 保存long类型值
	 * @param key
	 * @param value
	 */
	public void putLong(String key,long value)
	{
		Editor localEditor = mLocalPreferences.edit();
		localEditor.putLong(key, value);
		localEditor.commit();
	}
    
    /**
     * 存放String类型的数据
     * @param key
     * @param value
     */
    public void putString(String key, String value){
    	Editor localEditor = mLocalPreferences.edit(); 
    	localEditor.putString(key, value);
    	localEditor.commit(); 
    }

    /**
     * 存放Int类型的数据
     * @param key
     * @param value
     */
    public void putInt(String key, int value){
    	Editor localEditor = mLocalPreferences.edit(); 
    	localEditor.putInt(key, value);
    	localEditor.commit(); 
    }

	/**
     * 存放实体类以及任意类型
     * @param key
     * @param obj
     */
    public void putBean(String key, Object obj) {
        if (obj instanceof Serializable) {
        	// obj必须实现Serializable接口，否则会出问题
            try {
                ByteArrayOutputStream baos = new ByteArrayOutputStream();
                ObjectOutputStream oos = new ObjectOutputStream(baos);
                oos.writeObject(obj);
                String string64 = new String(Base64.encode(baos.toByteArray(),
                        0));
                Editor localEditor = mLocalPreferences.edit();
                localEditor.putString(key, string64);
                localEditor.commit();
            } catch (IOException e) {
            	DebugLog.LogE("put data to preference has exception:"+e.getMessage());
            }
        } else {
        	DebugLog.LogE("the obj must implement Serializble");
        }
    }

	/**
	 * 保存地理信息
	 * @param location
	 */
	public void putLocation(Location location)
	{
		if(location == null)
			return;
		Editor localEditor = mLocalPreferences.edit();
    	localEditor.putFloat(KEY_LATITUDE,(float)location.getLatitude());
    	localEditor.putFloat(KEY_LONGITUDE,(float)location.getLongitude());
    	localEditor.commit();
	}

	/**
     * 移除实体类以及任意类型
     * @param key
     */
    public void removeBean(String key) {
        try {
        	Editor editor = mLocalPreferences.edit();
            editor.remove(key).commit();
        } catch (Exception e) {
        	DebugLog.LogE("remove data from preference has exception:"+e.getMessage());
        }
    }
    

	/**
     * 获取实体类以及任意类型
     * @param key
     * @return obj
     */
    public Object getBean(String key, String defValue) {
        Object obj = null;
        try {
            String base64 = mLocalPreferences.getString(key, defValue);
            if (base64.equals("")) {
                return null;
            }
            byte[] base64Bytes = Base64.decode(base64.getBytes(), 1);
            ByteArrayInputStream bais = new ByteArrayInputStream(base64Bytes);
            ObjectInputStream ois = new ObjectInputStream(bais);
            obj = ois.readObject();
        } catch (Exception e) {
        	DebugLog.LogE("get data from preference has exception:"+e.getMessage());
        }
        return obj;
    }
	
		
	/**
	 * 获取boolean类型值
	 * @param key
	 * @param defValue
	 */
	public boolean getBoolean(String key,boolean defValue)
	{
		return mLocalPreferences.getBoolean(key, defValue);
	}
	
	/**
	 * 获取long类型值
	 * @param key
	 * @param defValue
	 */
	public long getLong(String key,long defValue)
	{
		return mLocalPreferences.getLong(key, defValue);
	}

    /**
     * 获取String类型的数据
     * @param key
     * @return value
     */
    public String getString(String key, String defvalue){
    	return mLocalPreferences.getString(key, defvalue);
    }

    /**
     * 获取Int类型的数据
     * @param key
     * @return value
     */
    public int getInt(String key, int defvalue){
    	return mLocalPreferences.getInt(key, defvalue);
    }
	
	
	/**
	 * 获取地理位置信息，一个小时更新一次
	 * 需要验证，不能主动开启GPS
	 * key:KEY_LATITUDE,KEY_LONGITUDE
	 * @return
	 */
    public synchronized float getLocation(String key)
    {

    	try
    	{    		
        	if( hasGpsPermission && 
        		System.currentTimeMillis() - loction_Update_Time > 216000)
        	{
    			LocationManager manager   = (LocationManager) mContext.getApplicationContext().getSystemService(Context.LOCATION_SERVICE);
        		loction_Update_Time = System.currentTimeMillis();
        		putLong(Config.KEY_LOCTION_LAST_UPDATE, loction_Update_Time);
        		
    			DebugLog.LogS("getLocation begin:"+System.currentTimeMillis());
  
    			Criteria criteria = new Criteria();  
    			criteria.setAccuracy(Criteria.ACCURACY_FINE);   
    	    	String bestProvider = manager.getBestProvider(criteria, true);
    	    	DebugLog.LogS("bestProvider:" + bestProvider);
    	    	Location location = manager.getLastKnownLocation(LocationManager.GPS_PROVIDER);
    	    	if(location != null)
    	    	{
    	    		DebugLog.LogD(location.toString());
    	    		putLocation(location);
    	    	}else
    	    	{
	    	    	location = manager.getLastKnownLocation(LocationManager.NETWORK_PROVIDER);
	    	    	if(location != null)
	    	    	{
	    	    		DebugLog.LogD(location.toString());
	    	    		putLocation(location);
	    	    	}
    	    	}
    			DebugLog.LogS("getLocation end:"+System.currentTimeMillis());
        	}
    	} 
	    catch (Exception e)
    	{
	    	// 由于多数应用无GPS权限，不显示异常log
//    		DebugLog.LogE(e);
    	}
    	
    	return mLocalPreferences.getFloat(key, -0.1f);
    }	
    
	/**
	 * 获取是否有GPS权限
	 * @param context
	 * @return
	 */
	public static boolean hasGPSPermission(Context context)
	{
		try {
			// 如果开发者不允许获取，直接返回失败
			if(!Setting.getLocationEnable())
				return false;
			if(context == null)
				return false;
			PackageInfo pkginfo = context.getPackageManager().getPackageInfo(
					context.getPackageName(),PackageManager.GET_PERMISSIONS);
			String[] permissions = pkginfo.requestedPermissions;
			for(int i = 0; i < permissions.length; i++)
			{
				if("android.permission.ACCESS_FINE_LOCATION".equalsIgnoreCase(permissions[i]) || 
					"android.permission.ACCESS_COARSE_LOCATION".equalsIgnoreCase(permissions[i]))
				return true;
			}
		} catch (Exception e) {
//			DebugLog.LogE(e);
		}
		return false;
	}
}
