package com.iflytek.cloud.msc.util.log;

import com.iflytek.msc.MSC;

public class DebugLog {

	private static String _TAG = "MscSpeechLog";
	
	/**
	 * <h4>共享库日志级别</h4>
	 * <p>共享库日志按重要程度区分了级别，目前log日志则没有相应的日志级别。</p>
	 * @see com.iflytek.cloud.Setting#setLogLevel
	 */
	public enum LOG_LEVEL
	{	
		/** 保存所有日志  */
		all,   
		/** 保存INFO等级日志 */
		detail,
		/** 保存DEBUG等级日志 */
		normal,  
		/** 保存ERR等级日志 */
		low,   
		/** 不保存日志*/
		none,    
	}
	
	/** <h4>是否打印日志（控制台和共享库）默认值</h4> */
	final public static boolean DEFAULT_IS_SHOW_LOG = true;

	// 日志级别
	private static LOG_LEVEL sLogLevel = LOG_LEVEL.normal;

	// 是否打印日志（控制台和共享库）
	private static boolean sIsShowLog   = DEFAULT_IS_SHOW_LOG;
	
	/**
	 * 是否允许打印msc内部测试日志
	 */
	private static boolean DEBUG_MODE = false;
	
	/**
	 * 设置log TAG
	 */
	public static void setTag(String tag)
	{
		_TAG = tag;
	}
	
	public static void LogD(String tag,String log)
	{
		if( isShowDebugLog() ){
			android.util.Log.d(tag, log);
		}//end of if show debug log
	}
	
	public static void LogD(String log)
	{
		LogD( _TAG, log );
	}
	
	public static void LogI( String tag, String log ){
		if( isShowInfoLog() ){
			android.util.Log.i(tag, log);
		}
	}
	
	public static void LogI( String log ){
		LogI( _TAG, log );
	}
	
	public static void LogE(String log)
	{
		LogE( _TAG, log );
	}
	
	public static void LogE(String tag,String log)
	{
		if( isShowErrorLog() ){
			android.util.Log.e(tag, log);
		}//end of if show error log
	}
	
	public static void LogE( Throwable exception ){
		if( isShowErrorLog() && null != exception ){
			exception.printStackTrace();
		}//end of show error log and exception not null
	}
	
	/**
	 * msc私有日志，正式版本不会进行打印
	 * @param log
	 */
	public static void LogS(String log)
	{
		LogS( _TAG, log );
	}
	
	public static void LogS(String tag,String log)
	{
		if( isShowSafeLog() ){
			android.util.Log.d(tag, log);
		}//end of if show safe log
	}
	
	public static void LogS(Throwable throwable)
	{
		if( isShowSafeLog() ){
			throwable.printStackTrace();
		}//end of if show safe log
	}
	
	public static void LogW( final String tag, final String log ){
		if( isShowErrorLog() ){
			android.util.Log.w( tag, log );
		}
	}
	
	public static void LogW( final String log ){
		LogW( _TAG, log );
	}
	
	public static boolean getShowLog(){
		return sIsShowLog;
	}
	
	public static void setShowLog( final boolean enable ){
		sIsShowLog = enable;
		updateJniLogStatus();
	}
	
	public static LOG_LEVEL getLogLevel(){
		return sLogLevel;
	}
	
	public static void setLogLevel( final LOG_LEVEL level ){
		sLogLevel = level;
		updateJniLogStatus();
	}
	
	private static boolean isShowErrorLog(){
		return getShowLog() 
				&& LOG_LEVEL.none!=getLogLevel();
	}
	
	private static boolean isShowDebugLog(){
		return getShowLog()
				&& ( getLogLevel().ordinal() <= LOG_LEVEL.normal.ordinal() );
	}
	
	private static boolean isShowInfoLog(){
		return getShowLog()
				&& ( getLogLevel().ordinal() <= LOG_LEVEL.detail.ordinal() );
	}
	
	private static boolean isShowSafeLog(){
		return DEBUG_MODE && getShowLog();
	}
	
	public static void updateJniLogStatus(){
		/**
		 * 设置jni中是否打印log
		 * 在{@link SpeechUtility}构造函数 中增加调用，防止调用本函数时，没有加载库而设置不了
		 */
		try {
			if( MSC.isLoaded() ){
				final boolean showJniLog = getShowLog() && isShowDebugLog();
				MSC.DebugLog( showJniLog );
				MSC.SetLogLevel( sLogLevel.ordinal() );
			}
		}catch( Throwable e ){
			DebugLog.LogD( "updateJniLogStatus exception: "+e.getLocalizedMessage() );
		}//end of try-catch
	}//end of function setShowJniLog
}
