package com.iflytek.cloud.param;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import android.text.TextUtils;

import com.iflytek.cloud.msc.util.log.DebugLog;

public class HashParam {
	
	// 参数值
	HashMap<String,String> kparam = new HashMap<String, String>();
	
	/**
	 * 解析以半角逗号为分割符的键值对，例：appid=123456,ent=sms16K
	 */
	public HashParam() {
		
	}
	
	
	/**
	 * 
	 * @param param 参数列表
	 * @param mapKeys 需要映射的关键字
	 */
	public HashParam(String param,String[][] mapKeys) {
		initWithParam(param);
		replaceKey(mapKeys);
	}
	
	/**
	 * 清空参数
	 */
	public void clear() {
		kparam.clear();
	}
	/**
	 * 初始化解析字符串
	 * @param param
	 */
	public void initWithParam(String param) {
		kparam.clear();
		putMultiParam(param);
	}
	
	/**
	 * 解析字符串
	 */
	public void putMultiParam(String param) {
		if (!TextUtils.isEmpty(param)) {
			String[] kvals = param.split(",");
			for(String val : kvals)
			{
				int index = val.indexOf("=");
				if(index > 0 && index < val.length())
				{
					String key = val.substring(0,index);
					String value = val.substring(index + 1);
					kparam.put(key, value);
				}
			}
		}
	}
	
	/**
	 * 复制对象
	 */
	@SuppressWarnings("unchecked")
	public HashParam clone(){
		HashParam ret = new HashParam();
		ret.kparam = (HashMap<String,String>)kparam.clone();
		return ret;
	}
	
		
	
	/**
	 * 插入参数，默认执行覆盖操作
	 * @param key
	 * @param value
	 */
	public void putParam(String key,String value) {
		putParam(key, value, true);
	}
	
	/**
	 * 插入param中使用的key-value
	 * @param param
	 * @param key
	 */
	public void putParam(HashParam param,String key)
	{
		if(param == null)
			return;
		putParam(key,param.getString(key));
	}
	public HashMap<String, String> getHash()
	{
		return kparam;
	}
	
	/**
	 * 插入Hash表数据
	 */
	public void putHash(HashParam param)
	{
		if(param != null)
			kparam.putAll(param.getHash());
	}
	
	/**
	 * 插入一条参数
	 * @param key key
	 * @param value  value
	 * @param replace 是否执行覆盖操作
	 */
	public void putParam(String key,String value,boolean replace) {
		if(TextUtils.isEmpty(key) || TextUtils.isEmpty(value))
			return;
		// 不执行覆盖操作
		if(!replace && kparam.containsKey(key))
			return;
		kparam.put(key, value);
	}
	
	/**
	 * 删除一条参数.
	 * @param key
	 */
	public Boolean removeParam(String key) {
		return null != kparam.remove(key);
	}
	
	/**
	 * 删除一条参数.
	 * @param key
	 */
	public String remove( String key ){
		return kparam.remove(key);
	}
	
	/**
	 * 返回以半角逗号进行分割的拼接字符串
	 */
	@Override
	public String toString() {
		//解决参数异常问题,由于有效率问题，只对AppinUtil做处理，不在此处调用trimParam
		StringBuffer ret = new StringBuffer();
		Iterator<Entry<String,String>> itors = kparam.entrySet().iterator();
		while(itors.hasNext())
		{
			Map.Entry<String,String> entry = itors.next();
			ret.append(entry.getKey());
			ret.append("=");
			ret.append(entry.getValue());
			ret.append(",");
		}
		if(ret.length() > 0)
			ret.deleteCharAt(ret.length()-1);
		// 打印隐私信息
		String rets = ret.toString();
		DebugLog.LogS(rets);
		
		return rets;
	}
	
	/**
	 * 返回字符串
	 * @param key
	 * @return
	 */
	public String getString(String key)
	{
		return kparam.get(key);
	}
	
	/**
	 * 返回字符串
	 * @param key
	 * @return
	 */
	public String getString(String key, String defaultVal)
	{
		String val = kparam.get(key);
		if (val == null)
		{
			val = defaultVal;
		}

		return val;
	}
	/**
	 * 获取int型值
	 * @param key
	 * @param defValue
	 * @return
	 */
	public int getInt(String key,int defValue)
	{
		return parseInt( kparam.get(key), defValue );
	}
	
	/**
	 * 获取 long 型值
	 * @param key
	 * @param defValue
	 * @return
	 */
	public long getLong(String key,long defValue)
	{
		return parseLong( kparam.get(key), defValue );
	}
	
	/**
	 * 获取bool型值
	 * @param key
	 * @param defValue
	 * @return
	 */
	public boolean getBoolean(String key,boolean defValue)
	{	
		return parseBoolean( kparam.get(key), defValue );
	}
	
	/**
	 * 将关键字进行修改
	 * @param mapKey 需要映射的列表
	 */
	public void replaceKey(String[][] mapKey)
	{
		if(mapKey == null)
			return;
		for(int i = 0; i < mapKey.length; i++)
		{
			String[] mkey = mapKey[i];
			if(kparam.containsKey(mkey[0]))
			{
				String value= kparam.get(mkey[0]);
				// 修改所有关键字
				kparam.remove(mkey[0]);
				for(int n = 1; n < mkey.length; n++)	
					kparam.put(mkey[n],value);
			}
		}
	}
	/**
	 * MSC参数值value，不允许出现", \n 空格"字符
	 * 只在AppInfoUtil中调用，因为有严重的效率问题
	 * @param value
	 * @return
	 */
	public void trimParam()
	{
		Iterator<Entry<String,String>> itors = kparam.entrySet().iterator();
		while(itors.hasNext())
		{
			Map.Entry<String,String> entry = itors.next();
			String value = entry.getValue();
			// 简单正则表达式
			value = trimInvalid(value);
			entry.setValue(value);
		}
	}
	
	/**
	 * 格式化字符串，防止异常字符出现
	 * @param value
	 * @return
	 */
	public static String trimInvalid(String value)
	{
		if(TextUtils.isEmpty(value))
			return null;
		return value.replaceAll("[,\n ]", "|");
	}
	
	/**
	 * 是否包含为key的参数
	 * @return
	 */
	public boolean hasKey(String key) {
		return kparam.containsKey(key);
	}
	
	/**
	 * 判断hashparam是否为空
	 * @return true表示hashparam为空.
	 */
	public boolean isEmpty() {
		return kparam.isEmpty();
	}

	public static boolean parseBoolean( final String strVal, final boolean defVal ){
		boolean ret = defVal;
		
		if(strVal != null)
		{
			if(strVal.equals("true") || strVal.equals("1") ){
				ret = true;
			}else if(strVal.equals("false") || strVal.equals("0") ){
				ret = false;
			}
		}
		
		return ret;
	}
	
	public static int parseInt( final String strVal, final int defVal ){
		int ret = defVal;
		
		if( strVal != null ){
			try{
				ret = Integer.parseInt( strVal );
			}catch(Exception e){
			}
		}
		
		return ret;
	}
	
	public static long parseLong( final String strVal, final long defVal ){
		long ret = defVal;
		
		if( strVal != null ){
			try{
				ret = Long.parseLong( strVal );
			}catch(Exception e){
			}
		}
		
		return ret;
	}
}
