/*
 * 文件名：SpeechTranscripterImpl.java 4.5.1049.1072 Sep 9 2014
 * 描述 ：转写对象具体实现.
 * 作者:iFLytek
 * 版权：科大讯飞
 */
package com.iflytek.cloud.speech.impl;

import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.TranscripterListener;
import com.iflytek.cloud.TranscripterResult;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.msc.ist.MscTranscripter;
import com.iflytek.cloud.msc.module.SessPerflog;
import com.iflytek.cloud.msc.module.SpeechModule;
import com.iflytek.cloud.msc.util.FuncAdapter;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.param.MscKeys;


public class SpeechTranscripterImpl extends SpeechModule{
	
	// 是否会话过程中暂停后台音乐播放
	private boolean mRequestFocus = false;
	public SpeechTranscripterImpl(Context context) {
		super(context);
	}

	/**
	 * 性能结构化日志插入
	 * @param key
	 */
	public void pushSessionInfo(String key) 
	{
		synchronized (mSynObj) {
			if(null != mscer)
				((MscTranscripter)mscer).getPerfLog().pushSessionInfo(key);
		}
	}
	
	/**
	 * 
	 * @param listener 回调监听
	 * @return void
	 */
	public int startTranscripting(TranscripterListener listener) {
		int ret = ErrorCode.SUCCESS;
		
		synchronized (mSynObj) {
			try{
				mRequestFocus = mSessionParams.getBoolean(SpeechConstant.KEY_REQUEST_FOCUS, true);
				
				mscer = new MscTranscripter(mContext,mSessionParams,getHandlerThread( "ist" ));
				//识别会话开始，暂停后台音乐播放.
				FuncAdapter.Lock(mContext, mRequestFocus, null);
				((MscTranscripter)mscer).startTranscripting(new InternalListener(listener));
			}catch(SpeechError e){
				ret = e.getErrorCode();
				DebugLog.LogE( e );
			}catch( Throwable e ){
				ret = ErrorCode.ERROR_UNKNOWN;
				DebugLog.LogE( e );
			}//end of try-catch
		}//end of synchronized
		
		return ret;
	}
	
	/**
	 * 写入录音数据
	 * @param buffer
	 * @return
	 */
	public boolean writeAudio(byte[] buffer,int offset,int length) {
		synchronized (mSynObj) {
			if (mscer == null) {	
				DebugLog.LogD("writeAudio error, no active session.");
				return false;
			}
			
			if(buffer == null || buffer.length <=0) {
				DebugLog.LogD("writeAudio error,buffer is null.");
				return false;
			}
			
			if(buffer.length < (length + offset)) {
				DebugLog.LogD("writeAudio error,buffer length < length.");
				return false;
			}
			
			if(((MscTranscripter)mscer).getAudioSource() != MscKeys.VALUE_AUDIO_EXTERNAL)
				return false;
			
			((MscTranscripter)mscer).onRecordBuffer(buffer,offset,length);
			return true;	
		}
	}

	/**
	 * Stops listening for speech.
	 */
	public void stopTranscripting() {
		synchronized (mSynObj) {
			if (mscer != null) {
				((MscTranscripter)mscer).stopTranscripting(true);	
			}
		}
	}
	/**
	 * 取消消息，保存底层事件
	 */
	@Override
	public void cancel(boolean notifyError) {
		//识别会话取消，后台音乐继续播放.
		FuncAdapter.UnLock(mContext, mRequestFocus, null);
		super.cancel(notifyError);
	}

	private final class InternalListener implements TranscripterListener {
		private TranscripterListener mOutListener = null;
		private static final int MSG_ERROR = 0;
		private static final int MSG_AUDIO_BUFFER = 1;
		private static final int MSG_SPEECH_BEGIN = 2;
		private static final int MSG_SPEECH_END = 3;
		private static final int MSG_RESULT = 4;
		// EVENT自定义消息
		private static final int MSG_EVENT  = 6;
		private boolean isFirstResult = false;
		public InternalListener(TranscripterListener outlistener) {
			mOutListener = outlistener;
		}

		private Handler mUiHandler = new Handler(Looper.getMainLooper()) {
			@Override
			public void handleMessage(Message msg) {
				if (mOutListener == null)
					return;
				switch (msg.what) {
				case MSG_ERROR:
					mOutListener.onError((SpeechError) msg.obj);
					break;
				case MSG_AUDIO_BUFFER:
					mOutListener.onVolumeChanged(msg.arg1,(byte[])msg.obj);
					break;
				case MSG_SPEECH_BEGIN:
					mOutListener.onBeginOfSpeech();
					break;
				case MSG_SPEECH_END:
					mOutListener.onEndOfSpeech();
					break;
				case MSG_RESULT:
				{
					mOutListener.onResult((TranscripterResult) msg.obj,
											msg.arg1==1?true:false);
					if(false == isFirstResult)
					{
						pushSessionInfo(SessPerflog.KEY_UI_FRS);
						isFirstResult = true;
					}
					if(1 == msg.arg1)
					{
						pushSessionInfo(SessPerflog.KEY_UI_LRS);
					}
				}
					break;
				case MSG_EVENT:
					Message mbody = (Message)msg.obj;
					// 自定义消息数据
					mOutListener.onEvent(mbody.what,mbody.arg1,mbody.arg2,(Bundle)mbody.obj);
					break;
				default:
					break;
				}
				super.handleMessage(msg);
			}
		};

		@Override
		public void onVolumeChanged(int volume, byte[] data) {
			Message msg = mUiHandler.obtainMessage(MSG_AUDIO_BUFFER, volume, 0,data);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onEndOfSpeech() {
			Message msg = mUiHandler.obtainMessage(MSG_SPEECH_END, 0, 0, null);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onResult(TranscripterResult result, boolean isLast) {
			//当返回最后一个结果，判断是否需要保存音频或继续音乐播放
			if(isLast)
			{
				sessionEnd();
			}
			Message msg = mUiHandler.obtainMessage(MSG_RESULT, 
								isLast == true ? 1 : 0, 0, result);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onBeginOfSpeech() {
			DebugLog.LogD("onBeginOfSpeech");
			Message msg = mUiHandler.obtainMessage(MSG_SPEECH_BEGIN, 0, 0, null);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onError(SpeechError error) {
			sessionEnd();
			Message msg = mUiHandler.obtainMessage(MSG_ERROR, error);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onEvent(int eventType, int arg1, int arg2, Bundle bundle) {
			
			Message msgbody = new Message();
			msgbody.what = eventType;
			msgbody.arg1 = arg1;
			msgbody.arg2 = arg2;
			msgbody.obj  = bundle;
			
			Message tmsg = mUiHandler.obtainMessage(MSG_EVENT, 0,0, msgbody);
			mUiHandler.sendMessage(tmsg);
		}
		
		/**
		 * 会话结束
		 * 判断是否需要保存音频或继续音乐播放
		 * 该接口不对外开放.
		 */
		protected void sessionEnd()
		{
			//识别会话结束，后台音乐继续播放.
			FuncAdapter.UnLock(mContext, mRequestFocus, null);
		}
	}

	public boolean isTranscripting() {
		return isRunning();
	}
	
	public String getSessionID(){
		if( null!=mscer ){
			return mscer.getSessionID();
		}else{
			return null;
		}
	}
	
	public int getUploadBytes(){
		if( null!=mscer ){
			return ((MscTranscripter)mscer).getUploadBytes();
		}else{
			return 0;
		}
	}
	
	public int getSyncID(){
		if( null!=mscer ){
			return ((MscTranscripter)mscer).getSyncID();
		}else{
			return 0;
		}
	}
	
	public String getAudioPath(){
		if( null!=mscer ){
			return ((MscTranscripter)mscer).getAudioPath();
		}else{
			return null;
		}
	}
}
