package com.iflytek.cloud.ui;

import java.util.Locale;

import android.content.Context;

import com.iflytek.cloud.InitListener;
import com.iflytek.cloud.resource.Resource;
import com.iflytek.cloud.ui.control.MscDialog;

/**
 * <h3>识别对话框</h3>
 * <p>
 * 		识别对话框是识别时，SDK自带的，用于用户交互的对话框界面。关于无对话框识别，
 * 请参考{@link com.iflytek.cloud.SpeechRecognizer}。
 * </p>
 * <p>
 * 		调用{@link #show}显示对话框后，自动开始录音（当设置
 * {@link com.iflytek.cloud.SpeechConstant#AUDIO_SOURCE 音频源}为麦克风时），
 * 即相当于调用了{@link com.iflytek.cloud.SpeechRecognizer#startListening}。
 * 在录音过程中点击对话框内任意地方停止录音并等待结果返回，即相当于调用了
 * {@link com.iflytek.cloud.SpeechRecognizer#stopListening}。点击对话框外的
 * 任意地方取消识别，对话框自动消失，即相当于调用了
 * {@link com.iflytek.cloud.SpeechRecognizer#cancel}。
 * 可以调用{@link #setOnDismissListener}来设置对话框消失时的监听器。
 * </p>
 * <p>
 * 		因识别对话框，实际上还是调用了{@link com.iflytek.cloud.SpeechRecognizer}的单例
 * 进行识别会话（在首次通过{@link #RecognizerDialog 构造函数}创建实例时，自动创建
 * {@link com.iflytek.cloud.SpeechRecognizer}的单例），所以应用不能在识别对话框还在进
 * 行会话（对话框在显示）时，调用 {@link com.iflytek.cloud.SpeechRecognizer#startListening}
 * 开启新的会话。
 * </p>

 * @author <script>gShowAuthor();</script>
 * @since version:1073
 * @see com.iflytek.cloud.SpeechRecognizer
 * @see RecognizerDialogListener
 */
public class RecognizerDialog extends MscDialog {

	/**
	 * <h4>构造函数</h4>
	 * <p>
	 * 		初始化一个识别对话框。
	 * </p>
	 * @param context 上下文环境变量
	 * @param listener 初始化监听器
	 * 
	 * @see com.iflytek.cloud.SpeechRecognizer#createRecognizer
	 */
	public RecognizerDialog(Context context, InitListener listener) {
		super(context);
		mView = new RecognizerView(context, listener);
	}
	
	/**
	 * <h4>设置识别对话框监听器</h4>
	 * <p>
	 * 		通过监听器，获取识别状态和结果。
	 * </p>
	 * @param listener 识别对话框监听器
	 * @see RecognizerDialogListener
	 */
	public void setListener(RecognizerDialogListener listener) {
		((RecognizerView)mView).setResultListener(listener);
	}
	
	/**
	 * <h4>设置参数</h4>
	 * <p>
	 * 		请参考{@link com.iflytek.cloud.SpeechRecognizer#setParameter(String, String)}说明。
	 * </p>
	 * 
	 * @see com.iflytek.cloud.SpeechRecognizer#setParameter(String, String)
	 */
	public void setParameter(String key, String value) {
		((RecognizerView)mView).setParameter(key, value);
	}
	
	/**
	 * <h4>设置对话框语言</h4>
	 * <p>
	 * 		目前支持以下语言：
	 * <br>简体中文：{@link Locale#CHINA}；
	 * <br>繁体中文：{@link Locale#TRADITIONAL_CHINESE}；
	 * <br>美式英语：{@link Locale#US}；
	 * </p>
	 * @param lan 语言种类，见上文说明。
	 */
	public void setUILanguage(Locale lan)
	{
		Resource.setUILanguage(lan);
	}

	/**
	 * <h4>显示</h4>
	 * <p>
	 * 		显示对话框，并开始识别，即相当于调用了{@link com.iflytek.cloud.SpeechRecognizer#startListening}。
	 * 若当前{@link com.iflytek.cloud.SpeechConstant#AUDIO_SOURCE 音频源}为麦克风，
	 * 将自动开启录音；若{@link com.iflytek.cloud.SpeechConstant#AUDIO_SOURCE 音频源}为
	 * 写音频流，则应用可通过调用{@link com.iflytek.cloud.SpeechRecognizer#writeAudio}写
	 * 入音频数据。
	 * </p>
	 * <p>
	 * 		关于对话框显示过程的其他操作，参考类说明。
	 * </p>
	 * 
	 * @see #dismiss
	 * @see com.iflytek.cloud.SpeechRecognizer#writeAudio
	 * @see com.iflytek.cloud.SpeechConstant#AUDIO_SOURCE 
	 * @see com.iflytek.cloud.SpeechRecognizer#startListening
	 */
	public void show() {
		super.show();
	}
	
	/**
	 * <h4>消失</h4>
	 * <p>
	 * 		使弹出框消失，同时取消当前会话，即相当于点击了对话框外的其他地方，
	 * 或相当于调用了{@link com.iflytek.cloud.SpeechRecognizer#cancel}。
	 * 会话被取消后，未返回的结果将不再返回。
	 * </p>
	 * 
	 * @see #show
	 * @see com.iflytek.cloud.SpeechRecognizer#cancel
	 */
	public void dismiss() {
		super.dismiss();
	}
}
