package com.iflytek.cloud.util.code.impl;

import java.util.HashMap;
import java.util.Map;

import android.content.Context;
import android.text.TextUtils;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.param.HashParam;
import com.iflytek.cloud.util.AudioDetector;
import com.iflytek.msc.VAD;

public class AudioDetectorImplFixfrontVad extends AudioDetector {
	private static final int DEF_TIMEOUT = 20000;
	private static final int DEF_EARLY_START= 1;
	
	private static final Map<String, Integer> sName2ID = new HashMap<String, Integer>();
	private static final Map<String, Integer> sName2Def = new HashMap<String, Integer>();
	
	private static final long INVALID_HANDLE = 0;
	private static final int MSC_TRUE = 1;
	
	private HashParam mParams = new HashParam();
	private long mHandle = INVALID_HANDLE;
	private DetectorResult mResult = new DetectorResult();
	private VAD.VadData mVadData = new VAD.VadData();
	private byte[] mOriginData = new byte[MAX_BUF_LEN];
	private byte[] mDetectData = new byte[MAX_BUF_LEN+16]; //fetch返回数据 <= append数据+1
	
	private boolean mIsFirstSub = true;
	
	private static final long VAL_INFINITE = -1;
	private int mBytesOfPerSample = 2;
	private long mBytesOfSpeechTimeout = VAL_INFINITE;
	private long mBytesOfSpeech = 0;

	static{
		//初始化map表
		sName2ID.put( SpeechConstant.VAD_BOS, 0 );
		sName2ID.put( SpeechConstant.VAD_EOS, 1 );
		sName2ID.put( SUB_TIMEOUT, 3 );
		sName2ID.put( EARLY_START, 4 );
		
		sName2Def.put( SpeechConstant.VAD_BOS, DEF_BOS );
		sName2Def.put( SpeechConstant.VAD_EOS, DEF_EOS );
		sName2Def.put( SUB_TIMEOUT, DEF_TIMEOUT );
		sName2Def.put( EARLY_START, DEF_EARLY_START );
	}
	
	static public class Status{
		public final static int OK = 0;	// 0
		public final static int START = 5;
		public final static int PAUSE = 6;
		public final static int START_AND_PAUSE = 7;
		
		public final static int FINISH = 8;
		public final static int START_AND_FINISH = 9;
		public final static int RESPONSE_TIMEOUT= 10;
		public final static int BUFFER_EMPTY= 11;
	}
	
	public AudioDetectorImplFixfrontVad(Context context, String param) {
		super(context, param);
		
		DebugLog.LogD( "AudioDetector constructor enter, context: "+context+", param: "+param );
		
		mParams.initWithParam( param );
		
		try{
			mHandle = VAD.Initialize( mParams.getInt( SpeechConstant.SAMPLE_RATE, DEF_RATE) );
			DebugLog.LogD( "VAD Initialize ret: "+ mHandle );
		}catch( Throwable e ){
			DebugLog.LogE( "AudioDetector constructor exception" );
			DebugLog.LogE( e );
		}
		
		mVadData.wavData = mDetectData;
	}//end of function constructor

	@Override
	public boolean destroy() {
		DebugLog.LogD( "destroy enter" );
		boolean ret = true;
		
		synchronized( sSyncObj ){
			if( INVALID_HANDLE != mHandle ){
				try{
					VAD.Uninitialize( mHandle );
					DebugLog.LogD( "VAD Uninitialize" );
					mHandle = INVALID_HANDLE;
				}catch( Throwable e ){
					DebugLog.LogE( "destroy exception" );
					DebugLog.LogE( e );
					ret=false;
				}
				
			}//end of if handle is valid
		}//end of synchronized
		
		sInstance = null;
		
		DebugLog.LogD( "destroy leave" );
		return ret;
	}// end of function destroy

	@Override
	public void reset() {
		DebugLog.LogD( "reset enter" );

		synchronized( sSyncObj ){
			if( INVALID_HANDLE != mHandle ){
				try{
					VAD.Reset( mHandle );
					DebugLog.LogD( "VAD Reset" );
					
					mIsFirstSub = true;
					mBytesOfSpeech = 0;
				}catch( Throwable e ){
					DebugLog.LogE( "reset exception" );
					DebugLog.LogE( e );
				}
				
			}//end of if handle is valid
		}//end of synchronized

		DebugLog.LogD( "reset leave" );
	}// end of function reset

	@Override
	public DetectorResult detect(byte[] buffer, int offset, int length, boolean isLast) {
		synchronized( sSyncObj ){
			DebugLog.LogI( "detect enter, buffer: "+buffer+", offset: "+offset+", length: "+length+", isLast: "+isLast );
			
			try{
				this.resetResult();
				
				if( INVALID_HANDLE == mHandle ){
					DebugLog.LogE( "detect error: handle is invalid!" );
					mResult.error = ErrorCode.ERROR_ENGINE_INIT_FAIL;
				}else if( null==buffer 
						|| 0>=length 
						|| MAX_BUF_LEN<length
						|| 0>offset
						|| buffer.length-offset<length){
					 if( isLast ){
						int status = VAD.EndAudioData(mHandle);
						DebugLog.LogD( "VAD EndAudioData ret: "+status );
						updateStatus( status );
						if( ErrorCode.SUCCESS == mResult.error ){
							mResult.error = VAD.GetLastSpeechPos( mHandle, mVadData );
							DebugLog.LogD( "VAD GetLastSpeechPos ret: "+mResult.error );
							if( ErrorCode.SUCCESS == mResult.error ){
								updateResult();
							}//end of if is succeed
						}
					 }else{
						 mResult.error = ErrorCode.ERROR_INVALID_PARAM;
					 }//end of if-else is last
				}else{
					System.arraycopy( buffer, offset, mOriginData, 0, length );
					DebugLog.LogI( "buffer length: "+length );
					
					int ret = VAD.CalcVolumLevel( mHandle, mOriginData, length, mVadData );
					DebugLog.LogI( "VAD CalcVolumLevel ret: "+ ret );
					mResult.error = ret;
					
					if( ErrorCode.SUCCESS == mResult.error ){
						ret = VAD.AppendData( mHandle, mOriginData, length );
						DebugLog.LogI( "VAD AppendData ret: "+ret );
						
						//Count for current bytes of speech since the next detect call of find start
						if( !mIsFirstSub ){
							mBytesOfSpeech += length;
						}
						
						updateStatus( ret );
						if( ErrorCode.SUCCESS == mResult.error ){
							ret = VAD.FetchData( mHandle, mVadData );
							DebugLog.LogI( "VAD FetchData ret: "+ret );
							updateStatus( ret );
							
							if( ErrorCode.SUCCESS==mResult.error){
								if( DetectorResult.STATUS_EOS==mResult.status
										|| DetectorResult.STATUS_BOS==mResult.status
										|| isLast ){
									
									ret = VAD.EndAudioData( mHandle );
									DebugLog.LogD( "VAD EndAudioData ret: "+ret );
									updateStatus( ret );
									
									if( ErrorCode.SUCCESS == mResult.error ){
										ret = VAD.GetLastSpeechPos( mHandle, mVadData );
										DebugLog.LogD( "VAD GetLastSpeechPos ret: "+ret );
										mResult.error = ret;
									}
									
								}//end of status
								
								if( ErrorCode.SUCCESS == mResult.error ){
									updateResult();
								}//end of if is succeed
								
							}//end of if FetchData is succeed
						}//end of if append data succeed
					}//end of if CalcVolumLevel is succeed
				}//end of if-else is parameter valid
				
			}catch( UnsatisfiedLinkError  e){
				DebugLog.LogE( "detect exception" );
				DebugLog.LogE( e );
				this.resetResult();
				mResult.error = ErrorCode.ERROR_UNSATISFIED_LINK;
			}catch( Throwable e ){
				DebugLog.LogE( "detect exception" );
				DebugLog.LogE( e );
				this.resetResult();
				mResult.error = ErrorCode.ERROR_UNKNOWN;
			}//end of try-catch
		}//end of synchronized
		
		DebugLog.LogD( "detect leave" );
		return mResult;
	}

	@Override
	public void setParameter(String key, String value) {
		DebugLog.LogD( "setParameter enter, key: "+key+", value: "+value );
		synchronized( sSyncObj ){
			try{
				if( !TextUtils.isEmpty(key) && sName2ID.containsKey(key) ){
					if( !TextUtils.isEmpty(value) ){
						mParams.putParam( key, value );
					}else{
						mParams.remove( key );
					}//end of if-else is value not empty
					
					final int newValue = mParams.getInt( key, sName2Def.get(key) );
					final int keyValue = sName2ID.get( key );
					int ret = VAD.SetParam( mHandle, keyValue, newValue );
					DebugLog.LogD( "VAD SetParameter key="+keyValue+", value="+newValue+", ret: "+ret );
				}else{
					if( SpeechConstant.KEY_SPEECH_TIMEOUT.equalsIgnoreCase(key) ){
						long timeout = VAL_INFINITE;
						try{
							timeout = Long.parseLong( value );
						}catch( NumberFormatException e ){
							
						}
						
						DebugLog.LogD( "SetParameter speech timeout value:"+timeout );
						if( 0<timeout ){
							final int rate = mParams.getInt( SpeechConstant.SAMPLE_RATE, DEF_RATE);
							mBytesOfSpeechTimeout = rate * mBytesOfPerSample * timeout / 1000;
							DebugLog.LogD( "SetParameter BytesOfSpeechTimeout: "+mBytesOfSpeechTimeout );
						}else{
							mBytesOfSpeechTimeout = VAL_INFINITE;
						}
					}else{
						//如果key 和 value都是int型值
						final int keyValue = Integer.parseInt( key );
						final int newValue = Integer.parseInt( value );
						int ret = VAD.SetParam( mHandle, keyValue, newValue );
						DebugLog.LogD( "VAD SetParameter key="+keyValue+", value="+newValue+", ret: "+ret );
					}//end of if-else jar parameter
				}//end of if-else is key not empty
			}catch( Throwable e ){
				DebugLog.LogE( "setParameter exception" );
				DebugLog.LogE( e );
			}//end of try-catch
		}//end of synchronized
		
		DebugLog.LogD( "setParameter leave." );
	}//end of function setParameter

	private void resetResult(){
		mResult.buffer = null;
		mResult.end = 0;
		mResult.error = ErrorCode.SUCCESS;
		mResult.length = 0;
		
		mResult.offset = 0;
		mResult.quality = 0;
		mResult.start = 0;
		mResult.status = 0;
		
		mResult.sub = 0;
		mResult.voice = false;
		mResult.volume = 0;
		
		mVadData.audioQuality = 0;
		mVadData.endByte = 0;
		mVadData.endRemainFrameNum = 0;
		mVadData.firstOutByte = 0;
		
		mVadData.inSpeech = 0;
		mVadData.startByte = 0;
		mVadData.startRemainFrameNum = 0;
		mVadData.status = 0;
		
		mVadData.volumeLevel = 0;
		mVadData.waitPauseOrEnd = 0;
		mVadData.waitStart = 0;
		mVadData.wavData = mDetectData;
		
		mVadData.wavDataSize = 0;
	}

	private void updateResult(){
		mResult.buffer = mVadData.wavData;
		mResult.end = mVadData.endByte;
		mResult.length = mVadData.wavDataSize;
		mResult.offset = 0;
		
		mResult.quality = mVadData.audioQuality;
		mResult.start = mVadData.startByte;
		mResult.voice = MSC_TRUE==mVadData.inSpeech;
		mResult.volume = mVadData.volumeLevel;
	}
	
	private void updateStatus( int status ){
		switch( status ){
		case Status.OK:
		case Status.BUFFER_EMPTY:
			mResult.error = ErrorCode.SUCCESS;
			break;
		case Status.START:
			mResult.sub = DetectorResult.SUB_START;
			break;
		case Status.PAUSE:
			mResult.sub = DetectorResult.SUB_END;
			break;
		case Status.START_AND_PAUSE:
			mResult.sub = DetectorResult.SUB_STARTEND;
			break;
		case Status.FINISH:
			mResult.status = DetectorResult.STATUS_EOS;
			break;
		case Status.START_AND_FINISH:
			mResult.sub = DetectorResult.SUB_STARTEND;
			mResult.status = DetectorResult.STATUS_EOS;
			break;
		case Status.RESPONSE_TIMEOUT:
			mResult.status = DetectorResult.STATUS_BOS;
			break;
		default:
			mResult.error = status;
		}//end of switch
		
		//有可能首次就返回 sub_end，这时也应作为找到前端点
		if( mIsFirstSub
				&& DetectorResult.SUB_OK!=mResult.sub ){
			mIsFirstSub = false;
			
			if( DetectorResult.STATUS_OK==mResult.status ){
				mResult.status = DetectorResult.STATUS_START;
			}
		}//end of if find first sub
		
		//Checking whether speech timeout or not
		if( DetectorResult.STATUS_OK==mResult.status
				&& isSpeechTimeout() ){
			mResult.status = DetectorResult.STATUS_TIMEOUT;
		}
	}//end of function updateStatus
	
	private boolean isSpeechTimeout(){
		boolean ret = false;
		if( 0 < mBytesOfSpeechTimeout ){
			ret = mBytesOfSpeechTimeout<=mBytesOfSpeech;
		}
		
		return ret;
	}//end of function isSpeechTimeout
}
