package com.iflytek.cloud.util.contacts;
  
import android.content.Context;
import android.graphics.Bitmap;
import android.os.StatFs;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;

import com.iflytek.cloud.msc.util.log.DebugLog;

/**
 * 文件操作通用类
 * @author weiwei
 *
 */
public class FileManager {

    /**
     * 拷贝asset资源到目的路径
     * @param descPath
     * @param assetPath
     * @return
     */
    public static boolean copyAssetFile(Context context,String descPath, String assetPath) {
//    	DebugLog.LogD(ContactManagerImpl.TAG, "desc file = " + descPath + ", asset file = " + assetPath);
        
        boolean ret = true;
        InputStream in = null;
        FileOutputStream fos = null;
        
        try  {
            // 读取程序包中资源
            in = context.getAssets().open(assetPath);
            int len = in.available();
            byte[] buffer = new byte[len];
            in.read(buffer);
            // 转存程序包资源
            fos = new FileOutputStream(descPath);
            fos.write(buffer, 0, len);
        } catch (IOException e) {
            ret = false;
            DebugLog.LogE(e);
        } finally {
            try {
                if (in != null) {
                    in.close();
                }
                if (fos != null) {
                    fos.close();
                }
            } catch (IOException e1) {
                ret = false;
                e1.printStackTrace();
            }
        }
        
        return ret;
    }
	/**
	 * 文件压缩
	 * @param sourceDir 被压缩文件路径
	 * @param desZipFile 压缩后文件存放路径及文件名 压缩成功
	 * @return 返回true 压缩失败,返回false
	 */
    public static boolean zip(String sourceDir, String desZipFile) {
        OutputStream outStream = null;
        BufferedOutputStream bufOutStream = null;
        ZipOutputStream zipOutStream = null;
        boolean isSucess = false;

        try {
            File desPath = new File(desZipFile);
            if (!desPath.getParentFile().exists()) {
                desPath.getParentFile().mkdirs();
            }

            outStream = new FileOutputStream(desZipFile);
            bufOutStream = new BufferedOutputStream(outStream);
            zipOutStream = new ZipOutputStream(bufOutStream);
            File sourceFile = new File(sourceDir);

            // 判断源文件路径是否存在
            if (!sourceFile.exists()) {//不存在
                //关闭数据流
                try {// 关闭数据流
                    if (outStream != null) {
                        outStream.close();
                    }
                    if (bufOutStream != null) {
                        bufOutStream.close();
                    }
                    if (zipOutStream != null) {
                        zipOutStream.closeEntry();
                        zipOutStream.close();
                    }
                } catch (IOException e) {
                    DebugLog.LogE(e);
                }
                return false;
            } else {//存在
                if (sourceFile.isFile()) {
                    isSucess = zipFile(sourceFile, "", zipOutStream);
                    if (!isSucess) {
                        try {// 关闭数据流
                            if (outStream != null) {
                                outStream.close();
                            }
                            if (bufOutStream != null) {
                                bufOutStream.close();
                            }
                            if (zipOutStream != null) {
                                zipOutStream.closeEntry();
                                zipOutStream.close();
                            }
                        } catch (IOException e) {
                            DebugLog.LogE(e);
                        }
                        return false;
                    }
                } else {
                    // 递归调用zipFile压缩函数
                    File[] files = sourceFile.listFiles();
                    for (int i = 0; i < files.length; i++) {
                        isSucess = zipFile(files[i], "", zipOutStream);
                        if (!isSucess) {
                            //System.out.println("ERROR");
                            try {// 关闭数据流
                                if (outStream != null) {
                                    outStream.close();
                                }
                                if (bufOutStream != null) {
                                    bufOutStream.close();
                                }
                                if (zipOutStream != null) {
                                    zipOutStream.closeEntry();
                                    zipOutStream.close();
                                }
                            } catch (IOException e) {
                                DebugLog.LogE(e);
                                return false;
                            }
                            return false;
                        }
                    }
                }

                if (zipOutStream != null) {
                    zipOutStream.closeEntry();
                    zipOutStream.close();
                }
            }
        } catch (Exception e) {
            try {// 关闭数据流
                if (outStream != null) {
                    outStream.close();
                }
                if (bufOutStream != null) {
                    bufOutStream.close();
                }
                if (zipOutStream != null) {
                    zipOutStream.closeEntry();
                    zipOutStream.close();
                }
            } catch (IOException e1) {
                DebugLog.LogE(e);
            }

            return false;
        } finally {
            try {// 关闭数据流
                if (outStream != null) {
                    outStream.close();
                }
                if (bufOutStream != null) {
                    bufOutStream.close();
                }
            } catch (IOException e) {
                DebugLog.LogE(e);
            }
            if (!isSucess) {// 未压缩成功，删除压缩文件
                File temFile = new File(desZipFile);
                if (temFile.exists()) {
                    temFile.delete();
                }
            }
        }
        
        return true;
    }

    /**
     * 文件压缩函数
     * @param sourceFile 源文件
     * @param curPath zip 输出控制流
     * @param zipOutStream 文件当前路径(相对路径) 
     * @return 压缩成功，返回true 压缩失败,返回false
     */
    private static boolean zipFile(File sourceFile, String curPath, ZipOutputStream zipOutStream) {
        InputStream inputStram = null;
        byte[] buffer = new byte[1024];
        int length = 0;
        ZipEntry zipEntry = null;
        boolean isSucess = false;
        
        try {
            if (!sourceFile.isDirectory()) {// 压缩文件
                inputStram = new FileInputStream(sourceFile);
                zipEntry = new ZipEntry(curPath + sourceFile.getName());
                zipOutStream.putNextEntry(zipEntry);

                while ((length = inputStram.read(buffer)) != -1) {
                    zipOutStream.write(buffer, 0, length);
                }
                zipOutStream.closeEntry();
            } else {// 压缩目录

                File[] files = sourceFile.listFiles();
                for (int i = 0; i < files.length; i++) {
                    // 递归压缩，更新curPaths,必须以"/"结束，表示目录
                    isSucess = zipFile(files[i], curPath + sourceFile.getName() + "/", zipOutStream);
                    if (!isSucess) {
                        if (inputStram != null) {
                            try {
                                inputStram.close();
                            } catch (IOException ex) {
                                ex.printStackTrace();
                            }
                        }
                        return false;
                    }
                }
            }
        } catch (IOException ex) {
            //ex.printStackTrace();
            if (inputStram != null) {
                try {
                    inputStram.close();
                } catch (IOException ex1) {
                    ex.printStackTrace();
                }
            }
            return false;
        }
        
        return true;
    }

    /**
     * 解压函数
     * @param sourceFile 压缩包所在路径
     * @param destDir 解压目的路径 
     * @return 解压成功,返回true 解压失败,返回false
     */
	public static boolean unZip(String sourceFile, String destDir) {
        OutputStream outStream = null;
        BufferedOutputStream bufOutStream = null;
        InputStream inputStram = null;
        destDir = destDir.endsWith(File.separator) ? destDir : (destDir + File.separator);
        byte b[] = new byte[1024];
        int length;
        ZipFile readZipFile = null;

        try {
            readZipFile = new ZipFile(new File(sourceFile));
            @SuppressWarnings("rawtypes")
			Enumeration enumeration = readZipFile.entries();
            ZipEntry zipEntry = null;

            while (enumeration.hasMoreElements()) {
                zipEntry = (ZipEntry) enumeration.nextElement();
                File outPutFile = new File(destDir + zipEntry.getName());

                if (zipEntry.isDirectory()) {// 当前条目是目录
                    outPutFile.mkdirs();
                } else {// 当前条目是文件

                    if (!outPutFile.getParentFile().exists()) {// 父目录不存在，则创建
                        outPutFile.getParentFile().mkdirs();
                    }

                    outStream = new FileOutputStream(outPutFile);
                    bufOutStream = new BufferedOutputStream(outStream);
                    inputStram = readZipFile.getInputStream(zipEntry);

                    // 解压文件
                    while ((length = inputStram.read(b)) > 0) {
                        bufOutStream.write(b, 0, length);
                    }
                    bufOutStream.flush();

                }
            }
        } catch (IOException e) {
            DebugLog.LogE(e);
            try {
                if (outStream != null) {
                    outStream.close();
                }
                if (bufOutStream != null) {
                    bufOutStream.close();
                }
                if (inputStram != null) {
                    inputStram.close();
                }
                if (readZipFile != null) {
                    readZipFile.close();
                }
            } catch (IOException e1) {
                DebugLog.LogE(e);
            }
            return false;
        } finally {
            try {
                if (outStream != null) {
                    outStream.close();
                }
                if (bufOutStream != null) {
                    bufOutStream.close();
                }
                if (inputStram != null) {
                    inputStram.close();
                }
                if (readZipFile != null) {
                    readZipFile.close();
                }
            } catch (IOException e) {
                DebugLog.LogE(e);
            }
        }
        
        return true;
    }

    /**
     * 查询File剩余空间
     * 
     * @return
     */
    public static long getFileAvailableSize(File file) {
        try {
            StatFs sf = new StatFs(file.getPath());
            long blockSize = sf.getBlockSize();
            long availCount = sf.getAvailableBlocks();

            return availCount * blockSize;
        } catch (Exception e) {
//        	DebugLog.LogD(ContactManagerImpl.TAG, "getFileAvailableSize error: " + e.toString());
        }
        return 0;
    }

    /**
     * 从文件读取文本
     * @param fileName
     * @return
     */
    public static String readString(String fileName) {
        String ret = null;
        try {
            File f = new File(fileName);
            FileInputStream byteOut = new FileInputStream(f);
            int len = byteOut.available();
            byte[] buffer = new byte[len];
            byteOut.read(buffer);
            ret = new String(buffer, "utf-8");
            byteOut.close();
        } catch (IOException e) {
//        	DebugLog.LogD(ContactManagerImpl.TAG, "load file failed. " + fileName);
        }
        
        return ret;
    }

    /**
     * 保存文本到文件
     * @param fileName
     * @param text
     * @param isWipe 是否擦除旧内容
     * @return
     */
    public static int writeString(String fileName, String text, boolean isWipe) {
        int file_len = 0;
        try {
            File file = new File(fileName);

            if (!file.exists()) {
                file.createNewFile();
            } else {
                if (isWipe) {
                    file.delete();
                    file.createNewFile();
                }
            }
            RandomAccessFile tmp_file = new RandomAccessFile(file, "rw");
            tmp_file.seek(tmp_file.length());
            tmp_file.write(text.getBytes("utf-8"));
            file_len = (int) tmp_file.length();
            tmp_file.close();
        } catch (IOException e) {
//        	DebugLog.LogD(ContactManagerImpl.TAG, "save file failed. " + fileName);
            return file_len;
        } 
        return file_len;
    }

    /**
     * 检查文件长度
     * 
     * @return
     */
    public static int getFileLenth(String filename) {
        int file_len = 0;
        try {
            File file = new File(filename);
            if (!file.exists()) {
                return 0;
            }
            RandomAccessFile tmp_file = new RandomAccessFile(file, "r");
            file_len = (int) tmp_file.length();
            tmp_file.close();
        } catch (IOException e) {
//        	DebugLog.LogD(ContactManagerImpl.TAG, "getLogFileLenth file failed. " + filename);
            return file_len;
        }
        return file_len;
    }
    
    /**
     * 检查文件是否存在
     * 
     * @param sourceFilePath 文件路径
     * @return true 该路径下存在文件 false 该路径下不存在文件
     */
    public static boolean checkFileExist(String sourceFilePath) {
        if (null == sourceFilePath) {
     //   	DebugLog.LogD(ContactManagerImpl.TAG, "sourceFilePath is null");
            return false;
        }

        try {
            File file = new File(sourceFilePath);
            if (file.exists()) {
                if (file.isDirectory()) {
                    File[] files = file.listFiles();
                    if (files.length > 0) {
                        return true;
                    }
                } else {
                    return true;
                }
            }
        } catch (Exception e) {
            DebugLog.LogE(e);
        }
        return false;
    }
    
    /**
     * 从指定目录中获取所有文件的文件名
     * @param sourceFilePath 指定文件路径
     * @return 所有文件文件名
     */
    public static List<String> getFileNameListFromPath(String sourceFilePath) {
        if (null == sourceFilePath) {
        //	DebugLog.LogD(ContactManagerImpl.TAG, "sourceFilePath is null");
            return null;
        }
        List<String> fileNameList = new ArrayList<String>();
        try {
            File file = new File(sourceFilePath);
            if (file.exists()) {
                if (file.isDirectory()) {
                    File[] files = file.listFiles();
                    if (files.length > 0) {
                       for (File tmpFile : files) {
                           fileNameList.add(tmpFile.getName());
                       }
                    }
                } else {
                    fileNameList.add(file.getName());
                }
            }
        } catch (Exception e) {
            DebugLog.LogE(e);
        }
        
        return fileNameList;
    }
    
    /**
     * 删除指定路径的文件或者文件夹
     */
    public static void deleteFileFromPath(String sourceFilePath) {
        if (null == sourceFilePath) {
        //	DebugLog.LogD(ContactManagerImpl.TAG, "sourceFilePath is null");
            return;
        }
        
        try {
            File file = new File(sourceFilePath);
            if (file.exists()) {
                if (file.isDirectory()) {
                    deleteDirectory(sourceFilePath);
                } else {
                    deleteFile(sourceFilePath);
                }
            }
        } catch (Exception e) {
            DebugLog.LogE(e);
        }
    }
    
    /**
     * 删除指定路径的文件
     * @param filePath
     */
    private static boolean deleteFile(String filePath) {
        if (null == filePath) {
       // 	DebugLog.LogD(ContactManagerImpl.TAG, "filePath is null");
            return false;
        }
        boolean flag = false;
        try {
            File file = new File(filePath);
            if (file.exists() && file.isFile()) {
                flag = file.delete();
            }
        } catch (Exception e) {
            DebugLog.LogE(e);
        }
        return flag;
    }
    
    /**
     * 删除指定路径的文件夹
     * @param directoryPath 文件夹路径
     * @return 删除是否成功
     */
    private static boolean deleteDirectory(String directoryPath) {
        if (!directoryPath.endsWith(File.separator)) {  
            directoryPath = directoryPath + File.separator;  
        }
        
        File dirFile = new File(directoryPath);  
        //如果dir对应的文件不存在，或者不是一个目录，则退出  
        if (!dirFile.exists() || !dirFile.isDirectory()) {  
            return false;  
        }  
        boolean flag = true;  
        //删除文件夹下的所有文件(包括子目录)  
        File[] files = dirFile.listFiles();  
        for (int i = 0; i < files.length; i++) {  
            //删除子文件  
            if (files[i].isFile()) {  
                flag = deleteFile(files[i].getAbsolutePath());  
                if (!flag) break;  
            } 
            //删除子目录  
            else {  
                flag = deleteDirectory(files[i].getAbsolutePath());  
                if (!flag) break;  
            }  
        }  
        if (!flag) return false;  
        //删除当前目录  
        if (dirFile.delete()) {  
            return true;  
        } else {  
            return false;  
        }  
    }
    
    /**
     * 从文件中删除前面的行
     * 
     * @param filename
     * @param del_line_count
     * @return line count;
     */
    public static int deleteLines(String filename, int del_line_count) {
        if (null == filename){
            return 0;
        }
        try {
            File file = new File(filename);
            if (!file.exists()) {
                return 0;
            }            
            RandomAccessFile tmp_file = new RandomAccessFile(file, "rw");
            int line = 0;
            int ch = tmp_file.read();
            int tmp_pos = 0;
            int del_pos = -1;
            while (ch != -1) {
                tmp_pos++;
                if (ch == '\n') {
                    line++;
                    if (line == del_line_count) {
                        del_pos = tmp_pos;
                        break;
                    }
                }
                ch = tmp_file.read();
            }
            if (line < del_line_count){
                tmp_file.close();
                file.delete();
            }else if (del_line_count == 0 ){
                tmp_file.close();
            }else  if (del_pos > 0){
                tmp_file.seek(del_pos);
                int len = (int) tmp_file.length() - del_pos;
                byte[] buffer = new byte[len];
                tmp_file.read(buffer);
                tmp_file.close();
                // delete old.
                file.delete();
                // write other.
                tmp_file = new RandomAccessFile(file, "rw");
                tmp_file.write(buffer);
                tmp_file.close();
            }
        } catch (IOException e) {
    //    	DebugLog.LogD(ContactManagerImpl.TAG, "deleteLine file failed. " + filename);
        }
        return getFileLineCount(filename);
    }

    /**
     * 向文件中追加行
     * 
     * @param fileName
     * @param text
     * @param isWipe
     * @return line count
     */
    public static int appendLine(String filename, String text, boolean isWipe) {
        if (null == filename){
            return 0;
        }
        try {
            File file = new File(filename);
            if (!file.exists()) {
                file.createNewFile();
            } else {
                if (isWipe) {
                    file.delete();
                    file.createNewFile();
                }
            }
            RandomAccessFile tmp_file = new RandomAccessFile(file, "rw");
            tmp_file.seek(tmp_file.length());
            if (text != null && text.length() > 0) {
                tmp_file.write(text.getBytes("utf-8"));
            }
            tmp_file.close();
        } catch (IOException e) {
//        	DebugLog.LogD(ContactManagerImpl.TAG, "appendLine failed. " + filename);
        }
        return getFileLineCount(filename);
    }

    /**
     * 获取文件行数
     * 
     * @param filename
     * @return
     */
    public static int getFileLineCount(String filename) {
        if (null == filename){
            return 0;
        }
        int line = 0;
        try {
            File file = new File(filename);
            if (!file.exists()) {
                return 0;
            }
            RandomAccessFile tmp_file = new RandomAccessFile(file, "r");
            int ch = tmp_file.read();
            int old_ch = -1;
            while (ch != -1) {
                if (ch == '\n')
                    line++;
                old_ch = ch;
                ch = tmp_file.read();
            }
            if  (ch == -1 && old_ch > 0 && old_ch != '\n'){
                line++;
            }
            tmp_file.close();
        } catch (IOException e) {
//        	DebugLog.LogD(ContactManagerImpl.TAG, "getFileLineCount file failed. " + filename);
        }
        return line;
    }
    
    /**
     * 读取文件前面N行
     * @param filename
     * @param top_limit
     * @return
     */
    public static ArrayList<String> readLines(String filename,int top_limit) {
        ArrayList<String>  ret_list = new ArrayList<String> (); 
        if (top_limit <= 0){
            return ret_list;
        }
        try {
            File file = new File(filename);
            if (!file.exists()) {
                return ret_list;
            }
            RandomAccessFile tmp_file = new RandomAccessFile(file, "r");
            String tmp_str = tmp_file.readLine();
             
            while (null != tmp_str) {
                ret_list.add(tmp_str);
                if (ret_list.size() >= top_limit){
                    break;
                }
                tmp_str = tmp_file.readLine();
            } 
            tmp_file.close();
        } catch (IOException e) {
     //   	DebugLog.LogD(ContactManagerImpl.TAG, "readLines file failed. " + filename);
        }
        return ret_list;
    }
    
    /**
	 * 读取assets目录中的文件
	 * @param context   上下文
	 * @param filePath 文件路径
	 */
	public static InputStream getInputStream(Context context, String filePath) {
		InputStream is = null;
		try {
			is = context.getAssets().open(filePath);
		} catch (IOException e) {
			DebugLog.LogE(e);
		} catch (Exception e) {
			DebugLog.LogE(e);
		}
		return is;
	}
	
	/**
	 * 读取内存中的文件
	 * @param filePath 文件路径
	 */
	public static InputStream getInputStream(String filePath) {
		InputStream is = null;
		try {
			is = new FileInputStream(filePath);
        } catch (FileNotFoundException e) {
        	DebugLog.LogE(e);
        } catch (Exception e) {
			DebugLog.LogE(e);
		}
        return is;
	}
	
	/**
	 * 将内存中的drawable存储到文件
	 * 
	 * @param fileName
	 * @param drawable
	 * @return 保存的文件路径
	 */
	public static String saveDrawableToFile(String fileName, String filePath, Bitmap bitmap) {
		FileOutputStream fOut = null;
		if (null == filePath || null == fileName) {
			return null;
		}
		try {
			File file = null;
			String url = "";
			
			// 保存到file目录
			File fileDir = new File(filePath);
			if (!fileDir.exists()) {
				fileDir.mkdirs();
			}

			file = new File(filePath + fileName);
			
			if (!file.exists()) {
				fOut = new FileOutputStream(file);
				if (fileName.endsWith(".jpg")) {
					bitmap.compress(Bitmap.CompressFormat.JPEG, 75, fOut);
				} else {
					bitmap.compress(Bitmap.CompressFormat.PNG, 100, fOut);
				}
				fOut.flush();
				fOut.close();
				bitmap.recycle();
			}

			url = "file://" + file.getAbsolutePath() + "?rc"
					+ System.currentTimeMillis();
			return url;
		} catch (Exception e) {
			DebugLog.LogE(e);
			return null;
		}
	}
}
